#ifndef _CAVIMAGEPOOL
#define _CAVIMAGEPOOL

/*
 *
 * This is part of the ANL/MCS/FL CAVEav distribution.
 * written by Ivan R. Judson & Bob Olson
 *
 */

#include "CAVImageDescriptor.h"

/** This is the ImagePool class.  This is a ring buffer of ImageDescriptors
  which provides some number of ImageDescriptors supplying them in 
  round-robbin style to some object that wants to use frames.
  */

#define MAXFRAMES 10

class CAVImagePool
{
public:
  /**@name Methods */
  //@{

  /// This constructor builds the ring buffer

  CAVImagePool(int width = MAXWID, int height = MAXHEI, int depth = MAXDEP, 
	    int nframes = MAXFRAMES) {
    int i;

    // Set paramters
    _clientActive = 0;
    _nextImage = _currentSerial = -1;

    // Set the defaults
    _nImages = nframes;

    // Set all the used image sizes
    for(i = 0; i < _nImages; i++)
      {
	_images[i].SetSize(width, height, depth);
      }
  }

  /// This destructor destroys the ring buffer

  ~CAVImagePool(void) {
    }

  /// This method frees the current images and reallocates new the right size

  void Resize(int width, int height) {
    int i;
  
    for(i = 0; i < _nImages; i++)
      {
	_images[i].SetSize(width, height, _images[i].GetDepth());
      }
  }

  /// Another option for creating new images of a different size
  
  void Resize(int width, int height, int depth) {
    int i;
  
    for(i = 0; i < _nImages; i++)
      {
	_images[i].SetSize(width, height, depth);
      }
  }

  //@}

  /**@name Data */
  //@{
    
  /// Client is taking frames

  int _clientActive;

  /// Total size of the ImagePool in bytes
	  
  int _totalPoolSize;
    
  /// Total number of images in ImagePool

  int _nImages;

  /// Index of next available image in ImagePool

  int _nextImage;

  /// Serial number of current image in ImagePool

  int _currentSerial;

  /// Array of ImageDescriptors that actually make up the ImagePool

  CAVImageDescriptor _images[MAXFRAMES];

  //@}
};

#endif



