#ifndef _SHAREDOBJECT
#define _SHAREDOBJECT

/*
 *
 * This is part of the ANL/MCS/FL CAVEav distribution.
 * written by Ivan R. Judson
 *
 */

#include <cave_ogl.h>

/** This is the SharedObject class. CAVSharedObject utilizes the
    CAVEMalloc() and CAVEFree() functions to provide shared objects in
    CAVE applications. This is done by overriding the new and delete
    operators for all objects that are subclasses of the SharedObject.  */

class CAVSharedObject
{
public:
  /**@name Methods */
  //@{
  /// This operator takes the same arguments and has the same return types as the standard C++ new operator. It is overridden in order to provide shared objects via the CAVEMalloc library call.

  void *operator new(size_t size) {
    void *p;
	
    p = CAVEMalloc(size);

#ifdef DEBUG     
    fprintf(stderr, "allocating new shared obj size=%d ptr=%x\n", size, p);
#endif

    return p;
  }

  /// This operator takes the same arguments and has the same return types as the standard C++ delete operator.  It is overridden to provide shared object deletion via the CAVEFree library call.

  void operator delete(void *p, size_t size) {
#ifdef DEBUG      
    fprintf(stderr, "Shared delete p=%x size=%d\n", p, size);
#endif

    CAVEFree(p);
  }
  //@}
};

#endif
