#ifndef _SHMCAVECAM
#define _SHMCAVECAM

/*
 *
 * This is part of the ANL/MCS/FL CAVEav distribution.
 * written by Ivan R. Judson
 *
 */

#include <GL/gl.h>
#include <GL/glu.h>
#include <stdio.h>
#include <stdlib.h>

#include "CAVcam.h"

/** This is the derived class shmCAVEcam.  shmCAVcam takes the frames
  captured and writes them out to a shared memory segment in rgba
  format.
  */

class shmCAVcam : public CAVcam {

public:
  /**@name  Methods */
  //@{
    
    /// Constructor with optional arguments for shared memory key, width, height, and depth 
	  
  shmCAVcam(int _shkey = 0, int width = 640, int height = 480, int depth = 4,
	     float ex = 0.0, float ey = 5.0, float ez = 0.0, 
	     float cx = 0.0, float cy = 5.0, float cz = -5.0,
	     float ux = 0.0, float uy = 1.0, float uz = 0.0);

    /// This destructor destroys the shared memory segment

  ~shmCAVcam();

    /// This function reads the renders the frame directly into the correct image descriptor in the ring buffer

  void Process(void);

    //@}
private:
  /**@name Data */
  //@{

  /// The shared memory key

  key_t _shmkey;

  /// The shared memory object 

  FLSharedMemory *_shmem;

  /// The temporary buffer for the frame until it's flipped directly into the shared memory ring buffer

  unsigned char *_tmpbuf;

//@}
};

#endif

